#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road,
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Nikolay Stanchev
##      Created Date :          17-04-2018
##      Created for Project :   FLAME
"""

import pytest
import time
import requests
import urllib.parse


class TestE2ESimulation(object):
    """
    A testing class used to group all the tests related to the E2E simulation data
    """

    @pytest.fixture(scope='class', autouse=True)
    def run_simulator(self, e2e_simulator):
        """
        A fixture, which runs the simulation before running the tests.

        :param e2e_simulator: the simulator for the end-to-end data
        """

        # Configure the aggregator through the CLMC service
        influx_url = urllib.parse.urlparse(e2e_simulator.db_url)
        aggregator_control_url = "http://{0}:9080/aggregator/control".format(influx_url.hostname)
        aggregator_config_url = "http://{0}:9080/aggregator/config".format(influx_url.hostname)

        print("Configuring aggregator with request to {0} ...".format(aggregator_config_url))
        r = requests.put(aggregator_config_url, json={"aggregator_report_period": 5, "aggregator_database_name": e2e_simulator.db_name, "aggregator_database_url": e2e_simulator.db_url})
        assert r.status_code == 200

        print("Running simulation, please wait...")
        e2e_simulator.run()

        print("Starting aggregator with request to {0}...".format(aggregator_control_url))
        r = requests.put(aggregator_control_url, json={"action": "start"})  # start the aggregator through the CLMC service
        assert r.status_code == 200

        print("Waiting for INFLUX to finish receiving data...")
        time.sleep(e2e_simulator.SIMULATION_LENGTH)  # wait for data to finish arriving at the INFLUX database
        print("... simulation data fixture finished")

        print("... stopping aggregator with request to {0}...".format(aggregator_control_url))
        r = requests.put(aggregator_control_url, json={"action": "stop"})  # stop the aggregator through the CLMC service
        assert r.status_code == 200

    @pytest.mark.parametrize("query, expected_result, equal_comparison", [
        ('SELECT count(*) FROM "CLMCMetrics"."autogen"."network_delays"',
         {"time": "1970-01-01T00:00:00Z", "count_latency": 120, "count_bandwidth": 120}, True),
        ('SELECT count(*) FROM "CLMCMetrics"."autogen"."service_delays"',
         {"time": "1970-01-01T00:00:00Z", "count_response_time": 24, "count_request_size": 24, "count_response_size": 24}, True),
        ('SELECT count(*) FROM "CLMCMetrics"."autogen"."e2e_delays"',
         {"time": "1970-01-01T00:00:00Z", "count_delay_forward": 40, "count_delay_reverse": 40, "count_delay_service": 40,
          "count_avg_request_size": 40, "count_avg_response_size": 40, "count_avg_bandwidth": 40}, False),
        ])
    def test_simulation(self, influx_db, query, expected_result, equal_comparison):
        """
        This is the entry point of the test. This method will be found and executed when the module is ran using pytest

        :param query: the query to execute (value obtained from the pytest parameter decorator)
        :param expected_result: the result expected from executing the query (value obtained from the pytest parameter decorator)
        :param influx_db the import db client fixture - imported from contest.py
        """

        # pytest automatically goes through all queries under test, declared in the parameters decorator
        print("\n")  # prints a blank line for formatting purposes

        # the raise_errors=False argument is given so that we could actually test that the DB didn't return any errors instead of raising an exception
        query_result = influx_db.query(query, raise_errors=False)

        # test the error attribute of the result is None, that is no error is returned from executing the DB query
        assert query_result.error is None, "An error was encountered while executing query {0}.".format(query)

        # get the dictionary of result points; the next() function just gets the first element of the query results generator (we only expect one item in the generator)
        actual_result = next(query_result.get_points())

        # check if we want to compare for equality or for '>='
        if equal_comparison:
            assert expected_result == actual_result, "E2E Simulation test failure"
        else:
            for key in expected_result:
                assert actual_result[key] >= expected_result[key], "E2E Simulation test failure"
